﻿-- Round a number to decimals
function TGTradeS:Round(num, idp)
  if not num then return end
  if idp and idp>0 then
    local mult = 10^idp;
    return math.floor(num * mult + 0.5) / mult;
  end
  return math.floor(num + 0.5);
end

-- GetItemInfo name Fix
function TGTradeS:GetItemInfos(iteminfo)
  local name, link, quality, iLevel,  reqLevel, class, subclass, maxStack,  equipSlot, texture, vendorPrice = GetItemInfo(iteminfo)
  if not name then
    return "Getting Item Info"
  end
  return name, link, quality, iLevel,  reqLevel, class, subclass, maxStack,  equipSlot, texture, vendorPrice
end

-- Convert GetProfessions into an table
function TGTradeS:GetProfessionList()
  local skills =  {};
  local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions();
  tinsert(skills, prof1);
  tinsert(skills, prof2);
  tinsert(skills, archaeology);
  tinsert(skills, fishing);
  tinsert(skills, cooking);
  tinsert(skills, firstAid);
  return skills;
end

-- CURRENCY Completer
function TGTradeS:CURRENCY()
  local id, needAmount = TourGuide.tags[TourGuide.current]:match("|CURRENCY|(%d+)%s?(%d*)|");
  needAmount = tonumber(needAmount) or 1;
  local name, amount, texture, unknown, weeklyMax, totalMax = GetCurrencyInfo(id)
  TGTradeS.extObj.text[1]:SetText(string.format("%i of %i %s", amount, needAmount, TGTradeS.currency[tonumber(id)]));
  if needAmount <= amount then
    TourGuide:SetTurnedIn(TourGuide.current, true);
    return true;
  end
  return false;
end

-- DONTHAVESKILL Completer
function TGTradeS:DONTHAVESKILL()
  local SkillName, SkillLevel = TourGuide.tags[TourGuide.current]:match("|DONTHAVESKILL|([%a%s]+)%s?(%d*)|");
  SkillName = strtrim(SkillName);
  SkillLevel = tonumber(SkillLevel) or 1;

  local ProfNum, noProfNum = 0, 0

  -- Loop all curent skill and check if the skill match
  for _, sID in ipairs(TGTradeS:GetProfessionList()) do
    if sID then
      ProfNum = ProfNum + 1
      local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)
      if SkillName == sName then
        if SkillLevel > tonumber(sLevel) then
          noProfNum = noProfNum + 1
        end
      else
        noProfNum = noProfNum + 1
      end
    end
  end
  if ProfNum == noProfNum then
    return false
  else
    TourGuide:SetTurnedIn(TourGuide.current, true);
    return true
  end
end

-- HAVESKILL Completer
function TGTradeS:HAVESKILL()
  local SkillName, SkillLevel = TourGuide.tags[TourGuide.current]:match("|HAVESKILL|([%a%s]+)%s?(%d*)|");
  SkillName = strtrim(SkillName);
  SkillLevel = tonumber(SkillLevel) or 1;

  -- Loop all curent skill and check if the skill match
  for _, sID in ipairs(TGTradeS:GetProfessionList()) do
    if sID then
      local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)
      if SkillName == sName and SkillLevel <= tonumber(sLevel) then
        return false;
      end
    end
  end
  TourGuide:SetTurnedIn(TourGuide.current, true);
  return true;
end

-- LEARNITEM Completer
function TGTradeS:LEARNITEM(msg)
  -- Get item name to learn
  local ItemName = TourGuide.tags[TourGuide.current]:match("|LEARNITEM|([%a:'%s]+)|");
  -- Check if there is a message containing the item name
  if msg then
    if msg:match("You have learned how to create a new item: ([%a:'%s]+).") then
      -- Get item nave of just learned
      local LearnedItemName = msg:match("You have learned how to create a new item: ([%a:'%s]+).");

      -- Compleate if match
      if (strtrim(ItemName) == strtrim(LearnedItemName)) then
        TourGuide:SetTurnedIn(TourGuide.current, true);
        return true;
      end
    end
  -- Check for allready known items
  else
    for _, Pvalue in pairs(TGTradeS.ProList) do
      if TGTradeS.db.char[Pvalue] then
        for _, RecipeName in pairs(TGTradeS.db.char[Pvalue]) do
          if RecipeName == strtrim(ItemName) then
            TourGuide:SetTurnedIn(TourGuide.current, true);
            return true;
          end
        end
      end
    end
  end
  return false;
end

-- LOOT Completer
function TGTradeS:LOOT()
  local info = { strsplit(",", TourGuide.tags[TourGuide.current]:match("|LOOT|([%d%s,]+)|") ) }
  local numDone, infoNum = 0, #info
  -- make sure the cap is max 6
  if infoNum > 6 then infoNum = 6 end

  for i = 1, infoNum, 1 do
    if strlen(strtrim(info[i])) > 0 then
      local ListInfo = strtrim(info[i])
      local lootitem, lootqty = ListInfo:match("(%d+)%s?(%d*)");
      lootqty = tonumber(lootqty) or 1;
      local itemName, itemLink = TGTradeS:GetItemInfos(tonumber(lootitem))

      TGTradeS.extObj.text[i]:SetText(string.format("%i of %i %s", GetItemCount(lootitem), lootqty, (itemLink or itemName)));

      -- Complete sub objective if it have all the amount
      if (GetItemCount(lootitem) >= lootqty) then
        numDone = numDone + 1
      end
    -- ensure that it will complete even that there is a , to mutch
    else
      numDone = numDone + 1
    end
  end
  if numDone >= infoNum then
    TourGuide:SetTurnedIn(TourGuide.current, true)
    return true
  else
    return false
  end
end

-- LVL Completer
function TGTradeS:LVL(level)
  lvl = tonumber(TourGuide.tags[TourGuide.current]:match("|LVL|(%d+)|"))
  if level then
    if lvl and lvl <= tonumber(level) then
      TourGuide:SetTurnedIn(TourGuide.current, true);
      return true;
    end
  else
    if lvl and lvl <= UnitLevel("Player") then
      TourGuide:SetTurnedIn(TourGuide.current, true);
      return true;
    end
  end
  return false;
end

-- LVLTOSKILL Completer
function TGTradeS:LVLTOSKILL()
  local SkillName, SkillLevel = TourGuide.tags[TourGuide.current]:match("|LVLTOSKILL|([%a:'%s]+)%s?(%d+)|");
  SkillName = strtrim(SkillName);
  SkillLevel = tonumber(SkillLevel) or 1;
  for _, sID in ipairs(TGTradeS:GetProfessionList()) do
    if sID then
      local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)
      if SkillName == sName then
        TGTradeS.extObj.text[1]:SetText(string.format("%s Skill %i of %i", SkillName, tonumber(sLevel), SkillLevel));
        if SkillLevel <= tonumber(sLevel) then
          TourGuide:SetTurnedIn(TourGuide.current, true);
          return true;
        end
      end
    end
  end
  return false;
end

-- MAP Level Completer
function TGTradeS:MAP()
  local info = TourGuide.tags[TourGuide.current]:match("|MAP|([%a%d:%s',_]+)|");
  if info then
    local _,_,MinLVL,MaxLVL = strsplit(",", info)
    -- Check if player are lower level than the map requires and complete if so, since we don't need to show it
    if MinLVL and UnitLevel("Player") < tonumber(strtrim(MinLVL)) then
      TourGuide:SetTurnedIn(TourGuide.current, true);
      return true
    end
    -- Check if player are higher level than the map requires and complete if so, since we don't need to show it
    if MaxLVL and UnitLevel("Player") > tonumber(strtrim(MaxLVL)) then
      TourGuide:SetTurnedIn(TourGuide.current, true);
      return true
    end
  end
  return false;
end

-- SKILL Completer
function TGTradeS:SKILL()
  local info = { strsplit(",", TourGuide.tags[TourGuide.current]:match("|SKILL|([%a%d%s,]+)|")) }
  local numDone = 0
  for i = 1, #info, 1 do

    if strlen(strtrim(info[i])) > 0 then
      local SkillName, SkillLevel = info[i]:match("([%a%s]+)%s?(%d*)");
      SkillName = strtrim(SkillName);
      SkillLevel = tonumber(SkillLevel) or 1;

      -- Loop all curent skill and check if the skill match
      for _, sID in ipairs(TGTradeS:GetProfessionList()) do
        if sID then
          local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)

          if SkillName == sName and SkillLevel <= tonumber(sLevel) then
            numDone = numDone + 1
          end
        end
      end
    -- ensure that it will complete even that there is a , to mutch
    else
      numDone = numDone + 1
    end
  end
  if numDone >= #info then
    TourGuide:SetTurnedIn(TourGuide.current, true)
    return true
  else
    return false
  end
end

-- TLOOT Completer
function TGTradeS:TLOOT()
  local itemListInfo, amount = TourGuide.tags[TourGuide.current]:match("|TLOOT|([%d%s]+),%s*(%d*)|")
  local lootqty = tonumber(amount) or 1
  
  local itemList = { strsplit(" ", strtrim(itemListInfo)) }
  
  local totalqty, infoNum = 0, #itemList
  -- make sure the cap is max 6
  if infoNum > 6 then infoNum = 6 end

  for i = 1, infoNum, 1 do
    if strlen(strtrim(itemList[i])) > 0 then
      local ListInfo = tonumber(strtrim(itemList[i]))
      local itemName, itemLink = TGTradeS:GetItemInfos(ListInfo)

      TGTradeS.extObj.text[i + 1]:SetText(string.format("%i x %s", GetItemCount(ListInfo), (itemLink or itemName)));

      totalqty = totalqty + GetItemCount(ListInfo)
    end
  end

  TGTradeS.extObj.text[1]:SetText(string.format("Get %i of either:", lootqty));
  if totalqty >= lootqty then
    TourGuide:SetTurnedIn(TourGuide.current, true)
    return true
  else
    return false
  end
end

-- TRAINSKILL Completer
function TGTradeS:TRAINSKILL(msg)
  local SkillName, MaxSkillLevel = TourGuide.tags[TourGuide.current]:match("|TRAINSKILL|([%a:'%s]+)%s?(%d*)|");
  SkillName = strtrim(SkillName);
  MaxSkillLevel = tonumber(MaxSkillLevel) or 75;
  if msg then
    local abilityname = strtrim(msg:match("You .+%[(.+)%]."));
    -- Get Skill info to check
    SkillName = strtrim(SkillName);
    -- If Ability name and skillname match
    if abilityname == SkillName then
      TourGuide:SetTurnedIn(TourGuide.current, true);
    end
  else
    for _, sID in ipairs(TGTradeS:GetProfessionList()) do
      if sID then
        local sName, _, _, maxSLevel, _, _, _, _ = GetProfessionInfo(sID)
        if SkillName == strtrim(sName) and MaxSkillLevel <= tonumber(maxSLevel) then
          TourGuide:SetTurnedIn(TourGuide.current, true);
          return true;
        end
      end
    end
  end
  return false;
end

-- Update Skill List
function TGTradeS:UPDATESKILLLIST(ProfessionName)
  if not tContains(TGTradeS.ProList, ProfessionName) then return end
  TGTradeS.db.char[ProfessionName] = {}
  local expanded, expIndex = {}, 1
  -- Expand all headers
  for index = GetNumTradeSkills(), 1, -1 do
    local skillName, skillType, numAvailable,  isExpanded, serviceType, numSkillUps  = GetTradeSkillInfo(index)
    if skillType == "header" then
      if not isExpanded then
        ExpandTradeSkillSubClass(index)
        expanded[expIndex] = skillName
        expIndex = expIndex + 1;
      end
    end
  end
  -- Collect Data and Collaps headers that was not expanded.
  for index = GetNumTradeSkills(), 1, -1 do
    local skillName, skillType, numAvailable,  isExpanded, serviceType, numSkillUps  = GetTradeSkillInfo(index)
    if skillType ~= "header" then
      tinsert(TGTradeS.db.char[ProfessionName], skillName)
      -- Remove Recipe from UnknownRecipe list if it's listed in there
      if #TGTradeS.db.char.UnknownRecipe > 0 then
        for URIndex = #TGTradeS.db.char.UnknownRecipe, 1, -1 do
          if TGTradeS.db.char.UnknownRecipe[URIndex] == skillName then
            tremove(TGTradeS.db.char.UnknownRecipe, URIndex)
          end
        end
      end
    else
      for i = 1, #expanded, 1 do
        if skillName == expanded[i] then
          CollapseTradeSkillSubClass(index)
        end
      end
    end
  end
end

-- Buy items from Merchant
function TGTradeS:AutoBuyFromMerchant()
  -- check config to see if player have chosen to auto buy items
  if TGTradeS.db.profile.Config.AutoBuy then
    local itemList = {}
    -- Check if the loot tag is formated correct
    if TourGuide.tags[TourGuide.current]:match("|LOOT|([%d%s,]+)|") then
      -- Get the item list to loot
      itemList = { strsplit(",", TourGuide.tags[TourGuide.current]:match("|LOOT|([%d%s,]+)|") ) }
      -- Drop the buy sequence if there is more than 6 items in the list.
      if #itemList > 6 then return end
    -- Drop the buy sequence if the loot tag is not formated corect
    else return end

    local money = GetMoney()

    -- loop throu the merchants items to check if he sells that item needed
    for i = 1, GetMerchantNumItems(), 1 do
      local Mname, _, price, _, numAvailable, _, extendedCost = GetMerchantItemInfo(i)

      for il = 1, #itemList, 1 do
        local lootitem, lootqty = itemList[il]:match("(%d+)%s?(%d*)")
        lootqty = tonumber(lootqty) or 1
        lootqty = lootqty - GetItemCount(lootitem)
        local Lname = TGTradeS:GetItemInfos(tonumber(lootitem))

        -- If merchant have the item, then initiate buy sequent
        if Mname == Lname and lootqty > 0 then
          -- If the item uses another currency check if player have it
          local haveCurrency = true
          if extendedCost then
            for ci = 1, GetMerchantItemCostInfo(i), 1 do
              _, value, link, name  = GetMerchantItemCostItem(i, ci)
              -- Check if it's an item to trade with and if player have the amount needed
              if haveCurrency and link then
                local itemType, itemid = link:match("|[%a%d]+|H(%a+):(%d+).*")
                local haveNum = GetItemCount(itemid)
                if haveNum < (value * lootqty) then
                  haveCurrency = false
                end
              end

              -- Check if it's an Reward to trade with and if player have the amount needed
              if haveCurrency and name then
                local _, amount = GetCurrencyInfo(TGTradeS.currency[name])
                if amount < (value * lootqty) then
                  haveCurrency = false
                end
              end
            end
          end

          -- check if player have the money and then buy it
          if (money >= (lootqty * price)) and haveCurrency then
            local BuyNum, maxStack = 0, GetMerchantItemMaxStack(i)
            local num = maxStack - (GetItemCount(lootitem) % maxStack)
            -- Fix num
            if num > lootqty then num = lootqty end
            -- If only merchant have a limited supply
            if numAvailable >= 0 then
              if numAvailable <= lootqty and numAvailable > 0 then
                BuyMerchantItem(i, numAvailable)
              elseif numAvailable > lootqty then
                BuyMerchantItem(i, lootqty)
              end
            -- If merchant have unlimited supply
            else
              if num <= lootqty then
                BuyMerchantItem(i, num)
              end

              while (num < lootqty) do
                if (num + maxStack) > lootqty then
                  BuyNum = lootqty - num
                else
                  BuyNum = maxStack
                end
                BuyMerchantItem(i, BuyNum)
                num = num + BuyNum
              end
            end
          else
            print(string.format("|cffff0000You don't have the needed currency to buy: %s|r", Lname))
          end
        end
      end
    end
  end
end

-- Auto Train Ability From Trainer
function TGTradeS:AutoTrainFromTrainer()
  -- check config to see if player have chosen to auto train from profession trainer
  if TGTradeS.db.profile.Config.AutoTrain and IsTradeskillTrainer() then
    local SkillName, SkillLevel = TourGuide.tags[TourGuide.current]:match("|TRAINSKILL|([%a:'%s]+)%s?(%d*)|");
    SkillName = strtrim(SkillName);
    SkillLevel = tonumber(SkillLevel) or 75
    local autoTrainName = tostring(TGTradeS.proTrainNameLevel[SkillLevel] .. " " .. TGTradeS.proTrainName[SkillName])

    -- Check if trainer have that ability you need.
    for index = 1, GetNumTrainerServices(), 1 do
      local serviceName, serviceSubText, serviceType,  isExpanded = GetTrainerServiceInfo(index)
      -- trainer have the ability so train it if you can
      if serviceName == autoTrainName then
        -- Train skill if available
        if serviceType == "available" then
          local moneyCost = GetTrainerServiceCost(index)
          if GetMoney() >= moneyCost then
            BuyTrainerService(index)
          else
            print(string.format("|cffff0000You don't have the needed money to buy: %s|r",autoTrainName))
          end
        -- Print out the message why you can train that skill
        else
          local _, _, hasReq = GetTrainerServiceSkillReq(index)
          if hasReq then
            local reqLevel = GetTrainerServiceLevelReq(index)
            if reqLevel and reqLevel > UnitLevel("Player") then
              print(string.format("|cffff0000You need to be level %i to train: %s|r", reqLevel, autoTrainName))
            else
              print(string.format("|cffff0000Somehow you can't train: %s|r",autoTrainName))
            end
          else
            print(string.format("|cffff0000Your skill level is too low to train: %s|r",autoTrainName))
          end
        end
      end
    end
  end
end

-- Auto Learn Recipe From Trainer
function TGTradeS:AutoLearnFromTrainer()
  -- check config to see if player have chosen to auto train from profession trainer
  if TGTradeS.db.profile.Config.AutoLearn and IsTradeskillTrainer() then
    -- Get item name to learn
    local autoLearnName = TourGuide.tags[TourGuide.current]:match("|LEARNITEM|([%a:'%s]+)|");

    -- Check if trainer have that ability you need.
    for index = 1, GetNumTrainerServices(), 1 do
      local serviceName, serviceSubText, serviceType,  isExpanded = GetTrainerServiceInfo(index)
      -- trainer have the ability so train it if you can
      if serviceName == autoLearnName then
        -- Train skill if available
        if serviceType == "available" then
          local moneyCost = GetTrainerServiceCost(index)
          if GetMoney() >= moneyCost then
            BuyTrainerService(index)
          else
            print(string.format("|cffff0000You don't have the needed money to buy recipe: %s|r",autoLearnName))
          end
        -- Print out the message why you can train that skill
        else
          local _, _, hasReq =  GetTrainerServiceSkillReq(index)
          if hasReq then
            local reqLevel = GetTrainerServiceLevelReq(index)
            if reqLevel and reqLevel > UnitLevel("Player") then
              print(string.format("|cffff0000You need to be level %i to learn recipe: %s!|r", reqLevel, autoLearnName))
            else
              print(string.format("|cffff0000Somehow you can't learn recipe: %s|r",autoLearnName))
            end
          else
            print(string.format("|cffff0000Your skill level is too low to learn recipe: %s|r",autoLearnName))
          end
        end
      end
    end
  end
end